@@ -18,6 +18,12 @@
 #   define _XOPEN_SOURCE 500
 #endif
 
+#if defined(__linux__) || defined(__CYGWIN__)
+#   ifndef _GNU_SOURCE
+#       define _GNU_SOURCE
+#   endif
+#endif
+
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -34,6 +40,14 @@
 #endif
 #include <sys/stat.h>
 #include <sys/types.h>
+#if defined(__linux__) || defined(__CYGWIN__)
+#include <dlfcn.h>
+#elif defined(__APPLE__)
+#define _DARWIN_C_SOURCE
+#include <dlfcn.h>
+#elif defined _WIN32
+#include <windows.h>
+#endif
 
 #ifndef DLL_UDUNITS2
 #define XML_STATIC
@@ -42,7 +56,7 @@
 #include "udunits2.h"
 
 #ifndef _XOPEN_PATH_MAX
-#   define _XOPEN_PATH_MAX 1024
+#   define _XOPEN_PATH_MAX 1024 // Includes terminating NUL
 #endif
 
 #define NAME_SIZE 128
@@ -97,7 +111,7 @@ static ut_status readXml(
 static File*            currFile = NULL;
 static ut_system*	unitSystem = NULL;
 static char*            text = NULL;
-static size_t           nbytes = 0;
+static size_t           nbytes = 0; /// Number of characters excluding NUL
 
 
 /*
@@ -117,7 +131,7 @@ ut_form_plural(
     const char*	plural = NULL;		/* failure */
 
     if (singular != NULL) {
-        int     length = strlen(singular);
+        size_t length = strlen(singular);
 
 	if (length + 3 >= sizeof(buf)) {
             ut_set_status(UT_SYNTAX);
@@ -2087,11 +2101,11 @@ readXml(
         }
 #else
         {
-            // str*cpy() must not copy overlapping strings
-            char tmp[_XOPEN_PATH_MAX];
-            (void)strncpy(tmp, path, sizeof(tmp));
-            tmp[sizeof(tmp)-1] = 0;
-            (void)strcpy(base, dirname(tmp));
+            // Temporary buffer used because `dirname()` modifies its argument
+            char tmp[strlen(path)+1];
+            (void)strcpy(tmp, path);
+            (void)strncpy(base, dirname(tmp), sizeof(base));
+            base[sizeof(base)-1] = 0;
         }
 #endif
 
@@ -2116,6 +2130,46 @@ readXml(
 }
 
 
+/* A bit hacky but much better than modifying binaries. */
+static const char*
+default_udunits2_xml_path()
+{
+    char const * soname = 0;
+#if defined(__APPLE__) || defined(__linux__) || defined(__CYGWIN__)
+    #define END_BIT "/share/udunits/udunits2.xml"
+    #define SEP '/'
+    Dl_info info;
+    if (dladdr(default_udunits2_xml_path, &info)) {
+        soname = info.dli_fname;
+    }
+#elif defined(_WIN32)
+    #define END_BIT "\\share\\udunits\\udunits2.xml"
+    #define SEP '\\'
+    // NB: XP+ solution!
+    HMODULE hModule = NULL;
+    GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
+            (LPCTSTR)default_udunits2_xml_path, &hModule);
+    char tmpbuf[MAX_PATH * 4];
+    GetModuleFileName(hModule, &tmpbuf[0], MAX_PATH * 4);
+    soname = &tmpbuf[0];
+#endif
+    if (soname) {
+        char * result = malloc(strlen(soname) + strlen(END_BIT) + 1);
+        if (result) {
+            strcpy(result, soname);
+            if (strrchr(result, SEP)) {
+                *strrchr(result, SEP) = '\0';
+                if (strrchr(result, SEP)) {
+                    *strrchr(result, SEP) = '\0';
+                }
+            }
+            strcat(result, END_BIT);
+            return result;
+        }
+    }
+    return DEFAULT_UDUNITS2_XML_PATH;
+}
+
 /**
  * Returns the pathname of the XML database.
  *
@@ -2129,7 +2183,7 @@ readXml(
  */
 const char*
 ut_get_path_xml(
-	const char*	path,
+	const char* path,
 	ut_status*  status)
 {
     if (path != NULL) {
@@ -2142,7 +2196,7 @@ ut_get_path_xml(
         	*status = UT_OPEN_ENV;
     	}
     	else {
-          	path = DEFAULT_UDUNITS2_XML_PATH;
+          	path = default_udunits2_xml_path();
         	*status = UT_OPEN_DEFAULT;
     	}
     }
